/****************************************************************************
 ***                                                                      ***
 ***   Rubens Sawaya Altimari                     Nmero USP: 3.286.287   ***
 ***   CCR - Corte Circular Restrito                         01/02/2004   ***
 ***   MAC-499                                                            ***
 ***   Supervisor: Ernesto G. Birgin                                      ***
 ***                                                                      ***
 ****************************************************************************/

#pragma hdrstop

#include <stdio.h>
#include <string.h>

#include "struts.h"
#include "ch.h"
#include "gabh.h"
#include "metapost.h"

#pragma argsused

/****************************************************************************
 ***   Defines e TypeDefs                                                 ***
 ****************************************************************************/

/****************************************************************************
 ***   Globals                                                            ***
 ****************************************************************************/

PHeuristica ResolveCCR = CH;     /* Pointer para heurstica (default: GH) */
FILE*       fLog       = NULL;   /* Arquivo .ccr (default: sem log)       */

/****************************************************************************
 ***   Local Prototypes                                                   ***
 ****************************************************************************/

int ChecaParametros(int argc, char* argv[]);

/****************************************************************************
 ***   main                                                               ***
 ***     Entrada: arquivo com dimenses da chapa e lista de peas         ***
 ***     Sada:   arquivo MetaPost (.mp) com distribuio de peas na     ***
 ***              chapa e arquivo ASCII (.ccr) com log de aes           ***
 ****************************************************************************/

int main(int argc, char* argv[])
{
  Chapa  C;      /* Chapa que ser cortada em crculos  */
  PChapa R = &C; /* Pointer para a chapa                */
  Pecas  I;      /* Conjunto de peas circulares        */
  int    N;      /* Nmero de peas circulares          */

  if (ChecaParametros(argc, argv))
  {
    PecasCarrega (argv[1], &C, &I, &N);
    ResolveCCR   (R, I, N);
    GeraMetapost (argv[1], R, I, N);
  }
  PecasDesaloca(I, N);
  if (fLog)
    fclose(fLog);

  return 0;
}

/****************************************************************************
 ***   ChecaParametros                                                    ***
 ***     Verifica se foram enviados nomes dos arquivos na linha de comando***
 ****************************************************************************/

int ChecaParametros(int argc, char* argv[])
{
  int  OK = TRUE;
  int  param;
  char nome[50];

  printf("\n***************************************************\n");
  printf(  "*  Rubens Sawaya Altimari       n.USP: 3.286.287  *\n");
  printf(  "*  MAC499 - CCR (Corte Circular Restrito)         *\n");
  printf(  "***************************************************\n\n");

  // Primeiro parmetro: nome do arquivo de entrada
  if (argc<2)
    OK = FALSE;
  else
  {
    // Opes: comea em argv[2] e verifica todas que comeam com '-'
    for (param = 2; param<argc && argv[param][0] == '-'; ++param)
    {
      switch (argv[param][1])
      {
        case 'h': // Heuristica
          if (argv[param][2] == 'G' || argv[param][2] == 'g')
            ResolveCCR = GABH;
          break;
        case 'd': // Debug
          strncpy(nome, argv[1], 46);
          strcat (nome, ".ccr");
          fLog = fopen(nome, "w");
          break;
        default: // parmetro invlido
          OK = FALSE;
          break;
      }
    }
  }

  if (!OK) {
    printf("Sintaxe: ccr <arquivo-de-pecas> <opcoes>\n\n");
    printf("         -hC: Heuristica Construtiva (default)\n");
    printf("         -hG: Heuristica Genetica\n");
    printf("         -d : Modo Debug (gera arquivo .CCR)\n");
    printf("\n");
  }
  return OK;
}

