/****************************************************************************
 ***                                                                      ***
 ***   Rubens Sawaya Altimari                     Nmero USP: 3.286.287   ***
 ***   CH - Contructive Heuristics                                        ***
 ***                                                                      ***
 ****************************************************************************/

#include <math.h>
#include <stdlib.h>

#include "ch.h"
#include "log.h"

/****************************************************************************
 ***   Defines e TypeDefs                                                 ***
 ****************************************************************************/

/****************************************************************************
 ***   Globals                                                            ***
 ****************************************************************************/

/****************************************************************************
 ***   Local Prototypes                                                   ***
 ****************************************************************************/

int Menor                (PPeca a, PPeca b);
int PecaCabeNaChapa      (PChapa R, PPeca p, PPonto t);
int PecaBateEmOutras     (PChapa R, Pecas I, int N, PPeca p, PPonto t);
int PosicaoViavel        (PChapa R, Pecas I, int N, PPeca p, PPonto t, char* desc);

/****************************************************************************
 ***   Menor                                                              ***
 ***     Compara pecas a e b, retornando TRUE se a deve vir antes de b    ***
 ***     Nesta heurstica, isto acontece quando a relao c/r for maior,  ***
 ***     e em caso de empate, se c for maior                              ***
 ****************************************************************************/

int Menor(PPeca a, PPeca b)
{
  double i = a->c/a->r, j = b->c/b->r;
  return (i > j || (IG(i,j) && a->c > b->c));
}
/****************************************************************************/
int PecaCabeNaChapa(PChapa R, PPeca p, PPonto t)
{
  if (MNIG(p->r, t->x) && MNIG(t->x, R->W - p->r) &&
      MNIG(p->r, t->y) && MNIG(t->y, R->L - p->r) )
  {
    return TRUE;
  }
  else {
    LogResultadoForaDaChapa();
    return FALSE;
  }
}
/****************************************************************************/
int PecaBateEmOutras(PChapa R, Pecas I, int N, PPeca p, PPonto t)
{
  PPeca s;
  int   bateu = FALSE;

  for (s = R->first; s != NULL; s = s->next)
  {
    if (s != p && !MNIG(QUAD(s->r + p->r), QDIST(s->o, (*t))))
    {
      LogResultadoBateu(s);
      bateu = TRUE;
      break;
    }
  }
  return bateu;
}
/****************************************************************************/
int PosicaoViavel(PChapa R, Pecas I, int N, PPeca p, PPonto t, char* desc)
{
  LogTentativaPonto(desc, t);
  if (PecaCabeNaChapa(R,p,t) && !PecaBateEmOutras(R,I,N,p,t)) {
    if (MNIG(QDIST0((*t)), QDIST0(p->o))) {
      LogResultadoOK();
      p->o = *t;
    }
    else
      LogResultadoMaisLonge();
    return TRUE;
  }
  return FALSE;
}

/****************************************************************************
 ***   Exported Functions                                                 ***
 ****************************************************************************/

/****************************************************************************
 ***   CH - Constructive Heuristics                                       ***
 ***     Implementao da heurstica construtiva, onde uma pr-ordenao  ***
 ***      feita em ordem no-crescente da relao c/r                    ***
 ****************************************************************************/

int CH(PChapa R, Pecas I, int N)
{
  LogChapa    (R, I, N);
  PecasOrdena (I, N, Menor);
  CA          (R, I, N);
  LogPecas    (R, I, N);

  return TRUE;
}

/****************************************************************************
 ***   CA - Constructive Approach                                         ***
 ***     Rotina para colocao de um cromossomo (ordenao de peas) na   ***
 ***     chapa. Utilizada tanto por CH quanto por GABH                    ***
 ****************************************************************************/

int CA(PChapa R, Pecas I, int N)
{
  PPeca  p, r, s;
  Ponto  q;
  PPonto t = &q;
  PAdj   l;
  double a, b, c, d, e;
  int    i;

  for (i = 0; i < N; ++i)
  {
    p = I + i;
    LogPeca(p);

    // posio inicial: fora da chapa, longe de (0,0)
    p->o.x = R->W + p->r;
    p->o.y = R->L + p->r;

    // posio trivial mais prxima de (0,0)
    t->x = p->r; t->y = p->r;
    PosicaoViavel(R, I, N, p, t, "trivial");

    // loop para cada pea j colocada na chapa - 4 posies adjacentes
    for (s = R->first; s != NULL; s = s->next)
    {
      // projeo (ortogonal) da distncia entre os centros
      double d = 2 * sqrt(s->r * p->r);

      LogTentativaPeca(s);

      // 4 posies, em relao a eixos  cima e  esquerda
      t->x = s->o.x - s->r + p->r; t->y = s->o.y - d;
      PosicaoViavel(R, I, N, p, t, "superior");
                                   t->y = s->o.y + d;
      PosicaoViavel(R, I, N, p, t, "inferior");
      t->x = s->o.x - d; t->y = s->o.y - s->r + p->r;
      PosicaoViavel(R, I, N, p, t, "esquerda");
      t->x = s->o.x + d;
      PosicaoViavel(R, I, N, p, t, "direita");
    }

    // loop para cada par de peas j colocadas na chapa
    for (r = R->first; r != NULL; r = r->next)
    {
      // lista de adjacncias da pea r
      for (l = r->adj; l != NULL; l = l->next)
      {
        s = l->peca;
        LogTentativaPecas(r, s);

        a = r->r + p->r;
        b = s->r + p->r;
        c = r->r + s->r;
        d = (QUAD(a)-QUAD(b)+QUAD(c))/(2*c);
        e = sqrt(QUAD(a)-QUAD(d));

        t->x = r->o.x + (d*(s->o.x - r->o.x) + e*(s->o.y - r->o.y)) / c;
        t->y = r->o.y + (d*(s->o.y - r->o.y) - e*(s->o.x - r->o.x)) / c;
        PosicaoViavel(R, I, N, p, t, "lado 1");
        t->x = r->o.x + (d*(s->o.x - r->o.x) - e*(s->o.y - r->o.y)) / c;
        t->y = r->o.y + (d*(s->o.y - r->o.y) + e*(s->o.x - r->o.x)) / c;
        PosicaoViavel(R, I, N, p, t, "lado 2");
      }
    }

    // atualizao da lista ligada de peas adjacentes
    if (p->o.x < R->W)
    {
      for (s = R->first; s != NULL; s = s->next)
      {
        if (IG(QDIST(p->o, s->o), QUAD(p->r + s->r)))
        {
          PAdj a = (PAdj) malloc(sizeof(Adj));
          a->peca = p;
          a->next = s->adj;
          s->adj  = a;
        }
      }
    }

    // atualizao da lista ligada de peas colocadas na chapa
    if (p->o.x < R->W)
    {
      if (R->first == NULL)
        R->first = p;
      else
        if (R->last != p)
          R->last->next = p;
      R->last = p;
    }
  }
  return TRUE;
}

