/****************************************************************************
 ***                                                                      ***
 ***   Rubens Sawaya Altimari                     Nmero USP: 3.286.287   ***
 ***   Rotinas para manipulao de MetaPost                               ***
 ***                                                                      ***
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "metapost.h"

/****************************************************************************
 ***   Defines e TypeDefs                                                 ***
 ****************************************************************************/

#define LARGURA 215.9 /* Tamanho do papel para sada, em mm (Letter) */
#define ALTURA  279.4
#define MARGEM   10.0

/****************************************************************************
 ***   Globals                                                            ***
 ****************************************************************************/

FILE* f = NULL;
struct {
  Ponto  o; /* coordenada da origem */
  double z; /* fator multiplicativo */
} Coord;

/****************************************************************************
 ***   Local Prototypes                                                   ***
 ****************************************************************************/

void GeraPonto     (double x, double y);
void GeraCirculo   (PPeca p);
void GeraDraw      (PPeca p);
void GeraFill      (PPeca p);
void GeraPeca      (PPeca p);
void GeraCabecalho (PChapa R);
void GeraRodape    (void);

/****************************************************************************
 ***   Local Functions                                                    ***
 ****************************************************************************/

void GeraPonto(double x, double y)
{
  fprintf(f, "(%20.10lfu,%20.10lfu)", Coord.o.x + x*Coord.z, Coord.o.y - y*Coord.z);
}
/****************************************************************************/
void GeraCirculo(PPeca p)
{
  GeraPonto(p->o.x - p->r, p->o.y       ); fprintf(f, "..\n     ");
  GeraPonto(p->o.x       , p->o.y - p->r); fprintf(f, "..\n     ");
  GeraPonto(p->o.x + p->r, p->o.y       ); fprintf(f, "..\n     ");
  GeraPonto(p->o.x       , p->o.y + p->r); fprintf(f, "..\n");
  fprintf(f, "cycle");
}
/****************************************************************************/
void GeraDraw(PPeca p)
{
  fprintf(f, "draw ");
  GeraCirculo(p);
  fprintf(f, ";\n");
}
/****************************************************************************/
void GeraFill(PPeca p)
{
  fprintf(f, "fill ");
  GeraCirculo(p);
  fprintf(f, " withcolor blue;\n");
}
/****************************************************************************/
void GeraPeca(PPeca p)
{
  GeraFill(p);
  GeraDraw(p);
  fprintf(f, "\n");
}
/****************************************************************************/
void GeraCabecalho(PChapa R)
{
  fprintf(f, "beginfig(1);\n\n");
  fprintf(f, "u = %lfmm;\n\n", 1.0);
  fprintf(f, "pickup pencircle scaled 3pt;\n\n");

  fprintf(f, "draw ");
  GeraPonto( 0.0,  0.0); fprintf(f, "--\n     ");
  GeraPonto(R->W,  0.0); fprintf(f, "--\n     ");
  GeraPonto(R->W, R->L); fprintf(f, "--\n     ");
  GeraPonto( 0.0, R->L); fprintf(f, "--\n");
  fprintf(f, "cycle;\n\n");

  fprintf(f, "pickup pencircle scaled 1pt;\n\n");
}
/****************************************************************************/
void GeraRodape(void)
{
  fprintf(f, "\nendfig;\n\n");
  fprintf(f, "end;\n");
}

/****************************************************************************
 ***   Exported Functions                                                 ***
 ****************************************************************************/

int GeraMetapost(char* arquivo, PChapa R, Pecas I, int N)
{
  int   i;
  char  nome[50];

  /* Sistema de Coordenadas */
  Coord.o.x = MARGEM;
  Coord.o.y = ALTURA - MARGEM;
  Coord.z   = R->L*LARGURA/R->W>ALTURA ? (ALTURA-2*MARGEM)/R->L : (LARGURA-2*MARGEM)/R->W;

  /* Arquivo de sada */
  strncpy(nome, arquivo, 46);
  strcat (nome, ".mp");
  if ((f = fopen(nome, "w")) != NULL)
  {
    PPeca p;

    GeraCabecalho(R);
    for (p = R->first; p != NULL; p = p->next)
      GeraPeca(p);
    GeraRodape();

    fclose(f); f = NULL;
    return TRUE;
  }
  printf("\nArquivo %s Invalido!\n\n", nome);
  return FALSE;
}

